/////////////////////////////////
//Automated Stagger Animation////
//Created by Weichen Wu//////////
//weichen.motion@gmail.com///////
/////////////////////////////////
function createDockableUI(thisObj) {
    var dialog = (thisObj instanceof Panel) ? thisObj : new Window("palette", "Automated UI Stagger Animation", undefined, { resizeable: true });
    
    // Set basic panel properties
    dialog.orientation = "column";
    dialog.alignChildren = ["fill", "top"];
    dialog.spacing = 10;
    dialog.margins = 16;

    // Add group for buttons with horizontal orientation
    var buttonGroup = dialog.add("group");
    buttonGroup.orientation = "row";  // Fixed horizontal orientation
    buttonGroup.alignChildren = ["fill", "center"];
    buttonGroup.spacing = 10;
    buttonGroup.margins = 0;

    // Add buttons
    var createBtn = buttonGroup.add("button", undefined, "Create the Controller");
    var applyBtn = buttonGroup.add("button", undefined, "Apply Animation");

    // Button click handlers
    createBtn.onClick = function() {
        if (!app.project.activeItem || !(app.project.activeItem instanceof CompItem)) {
            alert("Please select a composition first.");
            return;
        }

        var comp = app.project.activeItem;
        app.beginUndoGroup("Create Controller");

        try {
            var nullLayer = comp.layers.addNull();
            nullLayer.name = "Controller";

            // Move the controller to the bottom layer
            nullLayer.moveToEnd();

            // Add Slider Controls
            var yPosControl = nullLayer.effect.addProperty("ADBE Slider Control");
            yPosControl.name = "Y_Pos";
            yPosControl.property("Slider").setValue(50);

            var xPosControl = nullLayer.effect.addProperty("ADBE Slider Control");
            xPosControl.name = "X_Pos";
            xPosControl.property("Slider").setValue(0);

            var durationControl = nullLayer.effect.addProperty("ADBE Slider Control");
            durationControl.name = "Duration(frame)";
            durationControl.property("Slider").setValue(12);

            var delayControl = nullLayer.effect.addProperty("ADBE Slider Control");
            delayControl.name = "Delay(frame)";
            delayControl.property("Slider").setValue(2);

            var animOrderCheckbox = nullLayer.effect.addProperty("ADBE Checkbox Control");
            animOrderCheckbox.name = "Animation order: Top to Bottom";
            animOrderCheckbox.property("Checkbox").setValue(1);

        } catch (err) {
            alert("Error creating controller: " + err.toString());
        }

        app.endUndoGroup();
    };

    applyBtn.onClick = function() {
        if (!app.project.activeItem || !(app.project.activeItem instanceof CompItem)) {
            alert("Please select a composition first.");
            return;
        }

        var comp = app.project.activeItem;
        var selectedLayers = comp.selectedLayers;

        if (selectedLayers.length === 0) {
            alert("Please select at least one layer.");
            return;
        }

        app.beginUndoGroup("Apply Animation");

        try {
            for (var i = 0; i < selectedLayers.length; i++) {
                var layer = selectedLayers[i];
                var positionProp = layer.property("ADBE Transform Group").property("ADBE Position");

                // Check if position dimensions are separated
                if (positionProp.dimensionsSeparated) {
                    // X Position Expression
                    var xPositionExpression =
                        'var xPosChange = thisComp.layer("Controller").effect("X_Pos")("Slider");\n' +
                        'var duration_frames = thisComp.layer("Controller").effect("Duration(frame)")("Slider");\n' +
                        'var duration_seconds = thisComp.frameDuration * duration_frames;\n' +
                        'var stagger_delay = thisComp.layer("Controller").effect("Delay(frame)")("Slider") * thisComp.frameDuration;\n' +
                        'var animeOrder = thisComp.layer("Controller").effect("Animation order: Top to Bottom")("Checkbox").value;\n' +
                        'var totalLayers = thisComp.numLayers - 1;\n' +
                        'var layerIndex = (animeOrder == 1) ? index - 1 : totalLayers - index;\n' +
                        'var index_offset = layerIndex * stagger_delay;\n' +
                        'var startX = value + xPosChange;\n' +
                        'var endX = value;\n' +
                        'var t = time - (inPoint + index_offset);\n' +
                        'Math.easeOutExpo = function (t, b, c, d) {\n' +
                        '    t = clamp(t, 0, d);\n' +
                        '    return (t == d) ? b + c : c * (-Math.pow(2, -10 * t / d) + 1) + b;\n' +
                        '};\n' +
                        'Math.easeOutExpo(t, startX, endX - startX, duration_seconds);';
                    
                    // Y Position Expression
                    var yPositionExpression =
                        'var yPosChange = thisComp.layer("Controller").effect("Y_Pos")("Slider");\n' +
                        'var duration_frames = thisComp.layer("Controller").effect("Duration(frame)")("Slider");\n' +
                        'var duration_seconds = thisComp.frameDuration * duration_frames;\n' +
                        'var stagger_delay = thisComp.layer("Controller").effect("Delay(frame)")("Slider") * thisComp.frameDuration;\n' +
                        'var animeOrder = thisComp.layer("Controller").effect("Animation order: Top to Bottom")("Checkbox").value;\n' +
                        'var totalLayers = thisComp.numLayers - 1;\n' +
                        'var layerIndex = (animeOrder == 1) ? index - 1 : totalLayers - index;\n' +
                        'var index_offset = layerIndex * stagger_delay;\n' +
                        'var startY = value + yPosChange;\n' +
                        'var endY = value;\n' +
                        'var t = time - (inPoint + index_offset);\n' +
                        'Math.easeOutExpo = function (t, b, c, d) {\n' +
                        '    t = clamp(t, 0, d);\n' +
                        '    return (t == d) ? b + c : c * (-Math.pow(2, -10 * t / d) + 1) + b;\n' +
                        '};\n' +
                        'Math.easeOutExpo(t, startY, endY - startY, duration_seconds);';

                    try {
                        positionProp.property("ADBE Position_0").expression = xPositionExpression;
                        positionProp.property("ADBE Position_1").expression = yPositionExpression;
                    } catch (err) {
                        alert("Error setting separated position expressions: " + err.toString());
                    }
                } else {
                    // Unified position expression
                    var positionExpression =
                        'var yPosChange = thisComp.layer("Controller").effect("Y_Pos")("Slider");\n' +
                        'var xPosChange = thisComp.layer("Controller").effect("X_Pos")("Slider");\n' +
                        'var duration_frames = thisComp.layer("Controller").effect("Duration(frame)")("Slider");\n' +
                        'var duration_seconds = thisComp.frameDuration * duration_frames;\n' +
                        'var animeOrder = thisComp.layer("Controller").effect("Animation order: Top to Bottom")("Checkbox").value;\n' +
                        'Math.easeOutExpo = function (t, b, c, d) {\n' +
                        '    t = clamp(t, 0, d);\n' +
                        '    return (t == d) ? b + c : c * (-Math.pow(2, -10 * t / d) + 1) + b;\n' +
                        '};\n' +
                        'var stagger_delay = thisComp.layer("Controller").effect("Delay(frame)")("Slider") * thisComp.frameDuration;\n' +
                        'var totalLayers = thisComp.numLayers - 1;\n' +
                        'var layerIndex = (animeOrder == 1) ? index - 1 : totalLayers - index;\n' +
                        'var index_offset = layerIndex * stagger_delay;\n' +
                        'var startY = value[1] + yPosChange;\n' +
                        'var startX = value[0] + xPosChange;\n' +
                        'var endY = value[1];\n' +
                        'var endX = value[0];\n' +
                        'var t = time - (inPoint + index_offset);\n' +
                        'var newY = (yPosChange != 0) ? Math.easeOutExpo(t, startY, endY - startY, duration_seconds) : endY;\n' +
                        'var newX = (xPosChange != 0) ? Math.easeOutExpo(t, startX, endX - startX, duration_seconds) : endX;\n' +
                        '[newX, newY];';
                    
                    try {
                        positionProp.expression = positionExpression;
                    } catch (err) {
                        alert("Error setting unified position expression: " + err.toString());
                    }
                }

                // Apply opacity animation expression
                try {
                    var opacityProp = layer.property("ADBE Transform Group").property("ADBE Opacity");
                    var opacityExpression =
                        'var duration_frames = thisComp.layer("Controller").effect("Duration(frame)")("Slider");\n' +
                        'var duration_seconds = thisComp.frameDuration * duration_frames;\n' +
                        'var stagger_delay = thisComp.layer("Controller").effect("Delay(frame)")("Slider") * thisComp.frameDuration;\n' +
                        'var animeOrder = thisComp.layer("Controller").effect("Animation order: Top to Bottom")("Checkbox").value;\n' +
                        'var totalLayers = thisComp.numLayers - 1;\n' +
                        'var layerIndex = (animeOrder == 1) ? index - 1 : totalLayers - index;\n' +
                        'var index_offset = layerIndex * stagger_delay;\n' +
                        'var t = time - (inPoint + index_offset);\n' +
                        'linear(t, 0, duration_seconds, 0, 100);';
                    opacityProp.expression = opacityExpression;
                } catch (err) {
                    alert("Error setting opacity expression: " + err.toString());
                }
            }
        } catch (err) {
            alert("Error processing layers: " + err.toString());
        }

        app.endUndoGroup();
    };

    // Set minimum size
    dialog.minimumSize = [320, 100];

    // Show the panel
    if (dialog instanceof Window) {
        dialog.center();
        dialog.show();
    } else {
        dialog.layout.layout(true);
    }

    return dialog;
}

// Create and show the panel
var myPanel = createDockableUI(this);